clear variables; close all; clc;

% dimensions of matricees
noDimensions = 2;
xhat = [1; 0];
yhat = [0; 1];
I = [1, 0; 0, 1]; 

xhat(1); % indexing starts at 1

% how to get the dimension of a matrix
size(noDimensions);
size(xhat);
size(I);
length(xhat);

% the identity command
I = eye(noDimensions);

% vector arithmetic
beetlePosition = 3*xhat + yhat/0.5;

% dot products, in various forms
beetlePosition(1);
sum(beetlePosition .* xhat);
transpose(beetlePosition) * xhat;
dot(beetlePosition, xhat);
%beetlePosition * xhat;

% the transpose for real matrices
transpose(beetlePosition);
beetlePosition';
beetlePosition.';

% the .' operator and the complex conjugate
complexVector = [beetlePosition(1) + i * beetlePosition(2); 4 + 6i];
transposeCV = transpose(complexVector);
dotApostropheCV = complexVector.';
apostropheCV = complexVector';

% initializing matrices
lizardInitPos = zeros(noDimensions,1);
lizardInitVel = ones(noDimensions,1);
lizardInitAcc = 4 * (beetlePosition - lizardInitPos)/(norm(beetlePosition - lizardInitPos)^2 + 0.1);

% objects and storage
lizard.NoseToTailLength = 12.0e-2; %[m], 10 cm long
lizard.LegLength = 4.0e-2; %[m], 4 cm from toe to toe on one "axel". 
lizard.position = lizardInitPos; 
lizard.velocity = lizardInitVel;
lizard.acceleration = lizardInitAcc;

% the help function
%help atan2;

theta = atan2(lizard.velocity(1), lizard.velocity(2));

% matrix multiplication 
R = [sin(theta), -cos(theta); cos(theta), sin(theta)];
%disp(R); %display on screen
R*I;
R.*I;
R.'*R;

% accessing some variables
ehat1 = R(:,1);
ehat2 = R(:,2);
upperleft = R(1:1, 1);

% define some vectors to setup plotting
nose = R * [lizard.NoseToTailLength/2; 0] + lizard.position;
tail = R * [-lizard.NoseToTailLength/2; 0] + lizard.position;

leftFrontLeg = R * [0; -lizard.LegLength/2] + lizard.position + 0.35*lizard.NoseToTailLength*ehat1;
rightFrontLeg = R * [0; lizard.LegLength/2] + lizard.position + 0.35*lizard.NoseToTailLength*ehat1;

leftBackLeg = R * [0; -lizard.LegLength/2] + lizard.position - 0.25*lizard.NoseToTailLength*ehat1;
rightBackLeg = R * [0; lizard.LegLength/2] + lizard.position - 0.25*lizard.NoseToTailLength*ehat1;

% manually enter debug mode
%keyboard 

% manually continue from debug mode
%dbcont

% some more storage for plotting
xFrontLeg = [leftFrontLeg(1), rightFrontLeg(1)];
yFrontLeg = [leftFrontLeg(2), rightFrontLeg(2)];
BackLeg = [leftBackLeg(1), rightBackLeg(1); leftBackLeg(2), rightBackLeg(2)];

figgy = 1; figure(figgy); 
hold on;
plot(beetlePosition(1), beetlePosition(2), 'ro');
plot([tail(1), nose(1)], [tail(2), nose(2)], 'b');
plot(xFrontLeg, yFrontLeg, 'b');
plot(BackLeg(1,:), BackLeg(2,:), 'b');
legend('Beetle', 'Lizard', 'Location','southeast');
title('The lizard and the beetle');
xlabel('x [m]');
ylabel('y [m]'); 
axis equal
hold off; 
print('lizardAndBeetle','-depsc')

dt = 0.01; %[s]
% [ newPosition, newVelocity ] = takeStep(lizard.position, lizard.velocity, lizard.acceleration, dt);
% lizard.position = newPosition; 
% lizard.velocity = newVelocity; 
% 
% lizard.acceleration = 4 * (beetlePosition - lizard.position)/(norm(beetlePosition - lizard.position)^2 + 0.1);

% you can interrupt a program
% return 

% initialize the index counter and the history storage matrix
idx = 1;
posHistory = lizard.position;

% two conditions for the loop, joined by an "and" &&  ("or" would be || )
% 1) idx < 200: a maximum iteration count
% 2) norm(beetlePosition - lizard.position) > 0.01: the lizard has not yet caught the beetle
while(idx < 200 && norm(beetlePosition - lizard.position) > 0.01)
    
    % take our step -> see file takeStep.m
    [ newPosition, newVelocity ] = takeStep(lizard.position, lizard.velocity, lizard.acceleration, dt);
    lizard.position = newPosition; 
    lizard.velocity = newVelocity; 
    lizard.acceleration = 4 * (beetlePosition - lizard.position)...
        /(norm(beetlePosition - lizard.position)^2 + 0.1);
    
    % save the lizard's position in a new column after the last column
    posHistory(:,end+1) = lizard.position;
    
    pause(1.0e-5); % useful so that you can Ctrl-C out of an unsuccessful (or infinite) loop.
    
    idx = idx+1; % update the counter
end

% the same as previously, for final position.

theta = atan2(lizard.velocity(1), lizard.velocity(2));
R = [sin(theta), -cos(theta); cos(theta), sin(theta)];
ehat1 = R(:,1);
ehat2 = R(:,2);

nose = R * [lizard.NoseToTailLength/2; 0] + lizard.position;
tail = R * [-lizard.NoseToTailLength/2; 0] + lizard.position;

leftFrontLeg = R * [0; -lizard.LegLength/2] + lizard.position + 0.35*lizard.NoseToTailLength*ehat1;
rightFrontLeg = R * [0; lizard.LegLength/2] + lizard.position + 0.35*lizard.NoseToTailLength*ehat1;

leftBackLeg = R * [0; -lizard.LegLength/2] + lizard.position - lizard.NoseToTailLength/4*ehat1;
rightBackLeg = R * [0; lizard.LegLength/2] + lizard.position - lizard.NoseToTailLength/4*ehat1;

figgy = 2; figure(figgy); 
hold on;
plot(posHistory(1,:), posHistory(2,:), 'b--'); 
plot([tail(1), nose(1)], [tail(2), nose(2)], 'b');
plot([leftFrontLeg(1), rightFrontLeg(1)], [leftFrontLeg(2), rightFrontLeg(2)], 'b');
plot([leftBackLeg(1), rightBackLeg(1)], [leftBackLeg(2), rightBackLeg(2)], 'b');
plot(beetlePosition(1), beetlePosition(2), 'ro');
title('The path to the beetle');
xlabel('x [m]');
ylabel('y [m]'); 
axis equal
hold off; 
print('lizardPath','-depsc')
